import pytest
from rdflib.plugins.stores.sparqlstore import SPARQLStore

from test_utils import can_use_sparql_query, cannot_use_gsp_write, \
    cannot_use_sparql_update, can_use_gsp_read

DATASET_NAME = "ontologies"


class TestOntologiesDataset:
    def test_text_search(self, fuseki_url):
        endpoint = f"{fuseki_url}/{DATASET_NAME}"
        store = SPARQLStore(query_endpoint=endpoint)

        result = store.query("""
            PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT ?uri ?score ?label
            WHERE {
                (?uri ?score) text:query (rdfs:label 'ancient history') .
                ?uri rdfs:label ?label .
            }
            ORDER BY DESC(?score)
        """)
        assert len(result) == 14
        assert list(map(lambda item: str(item.label), list(result))) == [
            'Ancient History',
            'Ancient Cultures',
            'History',
            'Art History',
            'Medieval History',
            'Egyptology and Ancient Near Eastern Studies',
            'Early Modern History',
            'History of Philosophy',
            'History of Science',
            'Economic and Social History',
            'Modern and Current History',
            'General Education and History of Education',
            'Art History, Music, Theatre and Media Studies',
            'Architecture, Building and Construction History, Construction Research, Sustainable Building Technology'
        ]

        result = store.query("""
            PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
            PREFIX text: <http://jena.apache.org/text#>

            SELECT ?uri ?score ?label
            WHERE {
                (?uri ?score) text:query (skos:altLabel 'Alte Geschichte') .
                ?uri skos:altLabel ?label .
            }
            ORDER BY DESC(?score)
        """)
        assert len(result) == 6
        assert list(map(lambda item: str(item.label), list(result))) == [
            'Alte Geschichte',
            'Alte Kulturen',
            'Frühneuzeitliche Geschichte',
            'Mittelalterliche Geschichte',
            'Geschichte der Philosophie',
            'Neuere und Neueste Geschichte (einschl. Europäische Geschichte der Neuzeit und Außereuropäische Geschichte)'
        ]

    @pytest.mark.parametrize("endpoint_name", ["", "query", "sparql", "get"])
    def test_dataset_is_not_writeable_with_gsp(self, fuseki_url, endpoint_name):
        assert cannot_use_gsp_write(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["", "query", "sparql", "get"])
    def test_dataset_is_not_writeable_with_sparql_update(self, fuseki_url, endpoint_name):
        assert cannot_use_sparql_update(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["", "query", "sparql"])
    def test_can_use_sparql_query(self, fuseki_url, endpoint_name):
        assert can_use_sparql_query(fuseki_url, DATASET_NAME, endpoint_name)

    @pytest.mark.parametrize("endpoint_name", ["", "get"])
    def test_can_use_gsp_get(self, fuseki_url, endpoint_name):
        assert can_use_gsp_read(fuseki_url, DATASET_NAME, endpoint_name)
